"""
Copyright 2021 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from okta.models.authenticator\
    import Authenticator
from okta.utils import format_url
from okta.api_client import APIClient


class AuthenticatorClient(APIClient):
    """
    A Client object for the Authenticator resource.
    """

    def __init__(self):
        self._base_url = ""

    async def list_authenticators(
            self,
            keep_empty_params=False
    ):
        """
        Args:
        Returns:
            list: Collection of Authenticator instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authenticators
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Authenticator)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(Authenticator(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def get_authenticator(
            self, authenticatorId,
            keep_empty_params=False
    ):
        """
        Args:
            authenticator_id {str}
        Returns:
            Authenticator
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authenticators/{authenticatorId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Authenticator)

        if error:
            return (None, response, error)

        try:
            result = Authenticator(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_authenticator(
            self, authenticatorId, authenticator,
            keep_empty_params=False
    ):
        """
        Updates an authenticator
        Args:
            authenticator_id {str}
            {authenticator}
        Returns:
            Authenticator
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authenticators/{authenticatorId}
            """)

        if isinstance(authenticator, dict):
            body = authenticator
        else:
            body = authenticator.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Authenticator)

        if error:
            return (None, response, error)

        try:
            result = Authenticator(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def activate_authenticator(
            self, authenticatorId,
            keep_empty_params=False
    ):
        """
        Args:
            authenticator_id {str}
        Returns:
            Authenticator
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authenticators/{authenticatorId}/lifecycle
                /activate
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Authenticator)

        if error:
            return (None, response, error)

        try:
            result = Authenticator(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def deactivate_authenticator(
            self, authenticatorId,
            keep_empty_params=False
    ):
        """
        Args:
            authenticator_id {str}
        Returns:
            Authenticator
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/authenticators/{authenticatorId}/lifecycle
                /deactivate
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Authenticator)

        if error:
            return (None, response, error)

        try:
            result = Authenticator(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)



"""
Copyright 2020 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from urllib.parse import urlencode
from okta.models.inline_hook\
    import InlineHook
from okta.models.inline_hook_response\
    import InlineHookResponse
from okta.utils import format_url
from okta.api_client import APIClient


class InlineHookClient(APIClient):
    """
    A Client object for the InlineHook resource.
    """

    def __init__(self):
        self._base_url = ""

    async def list_inline_hooks(
            self, query_params={},
            keep_empty_params=False
    ):
        """
        Args:
            query_params {dict}: Map of query parameters for request
            [query_params.type] {str}
        Returns:
            list: Collection of InlineHook instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/inlineHooks
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, InlineHook)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(InlineHook(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def create_inline_hook(
            self, inline_hook,
            keep_empty_params=False
    ):
        """
        Args:
            {inline_hook}
        Returns:
            InlineHook
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/inlineHooks
            """)

        if isinstance(inline_hook, dict):
            body = inline_hook
        else:
            body = inline_hook.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, InlineHook)

        if error:
            return (None, response, error)

        try:
            result = InlineHook(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_inline_hook(
            self, inlineHookId,
            keep_empty_params=False
    ):
        """
        Deletes the Inline Hook matching the provided id. Once
        deleted, the Inline Hook is unrecoverable. As a safety
        precaution, only Inline Hooks with a status of INACTIVE
        are eligible for deletion.
        Args:
            inline_hook_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/inlineHooks/{inlineHookId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def get_inline_hook(
            self, inlineHookId,
            keep_empty_params=False
    ):
        """
        Gets an inline hook by ID
        Args:
            inline_hook_id {str}
        Returns:
            InlineHook
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/inlineHooks/{inlineHookId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, InlineHook)

        if error:
            return (None, response, error)

        try:
            result = InlineHook(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_inline_hook(
            self, inlineHookId, inline_hook,
            keep_empty_params=False
    ):
        """
        Updates an inline hook by ID
        Args:
            inline_hook_id {str}
            {inline_hook}
        Returns:
            InlineHook
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/inlineHooks/{inlineHookId}
            """)

        if isinstance(inline_hook, dict):
            body = inline_hook
        else:
            body = inline_hook.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, InlineHook)

        if error:
            return (None, response, error)

        try:
            result = InlineHook(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def execute_inline_hook(
            self, inlineHookId, inline_hook_payload,
            keep_empty_params=False
    ):
        """
        Executes the Inline Hook matching the provided inlineHo
        okId using the request body as the input. This will sen
        d the provided data through the Channel and return a re
        sponse if it matches the correct data contract. This ex
        ecution endpoint should only be used for testing purpos
        es.
        Args:
            inline_hook_id {str}
            {inline_hook_payload}
        Returns:
            InlineHookResponse
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/inlineHooks/{inlineHookId}/execute
            """)

        if isinstance(inline_hook_payload, dict):
            body = inline_hook_payload
        else:
            body = inline_hook_payload.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, InlineHookResponse)

        if error:
            return (None, response, error)

        try:
            result = InlineHookResponse(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def activate_inline_hook(
            self, inlineHookId,
            keep_empty_params=False
    ):
        """
        Activates the Inline Hook matching the provided id
        Args:
            inline_hook_id {str}
        Returns:
            InlineHook
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/inlineHooks/{inlineHookId}/lifecycle
                /activate
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, InlineHook)

        if error:
            return (None, response, error)

        try:
            result = InlineHook(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def deactivate_inline_hook(
            self, inlineHookId,
            keep_empty_params=False
    ):
        """
        Deactivates the Inline Hook matching the provided id
        Args:
            inline_hook_id {str}
        Returns:
            InlineHook
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/inlineHooks/{inlineHookId}/lifecycle
                /deactivate
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, InlineHook)

        if error:
            return (None, response, error)

        try:
            result = InlineHook(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)



"""
Copyright 2021 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from okta.models.brand\
    import Brand
from okta.models.theme_response\
    import ThemeResponse
from okta.models.image_upload_response\
    import ImageUploadResponse
from okta.utils import format_url
from okta.api_client import APIClient


class BrandClient(APIClient):
    """
    A Client object for the Brand resource.
    """

    def __init__(self):
        self._base_url = ""

    async def list_brands(
            self,
            keep_empty_params=False
    ):
        """
        List all the brands in your org.
        Args:
        Returns:
            list: Collection of Brand instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Brand)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(Brand(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def get_brand(
            self, brandId,
            keep_empty_params=False
    ):
        """
        Fetches a brand by `brandId`
        Args:
            brand_id {str}
        Returns:
            Brand
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Brand)

        if error:
            return (None, response, error)

        try:
            result = Brand(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_brand(
            self, brandId, brand,
            keep_empty_params=False
    ):
        """
        Updates a brand by `brandId`
        Args:
            brand_id {str}
            {brand}
        Returns:
            Brand
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}
            """)

        if isinstance(brand, dict):
            body = brand
        else:
            body = brand.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Brand)

        if error:
            return (None, response, error)

        try:
            result = Brand(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def list_brand_themes(
            self, brandId,
            keep_empty_params=False
    ):
        """
        List all the themes in your brand
        Args:
            brand_id {str}
        Returns:
            list: Collection of ThemeResponse instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, ThemeResponse)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(ThemeResponse(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def get_brand_theme(
            self, brandId, themeId,
            keep_empty_params=False
    ):
        """
        Fetches a theme for a brand
        Args:
            brand_id {str}
            theme_id {str}
        Returns:
            ThemeResponse
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes/{themeId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, ThemeResponse)

        if error:
            return (None, response, error)

        try:
            result = ThemeResponse(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_brand_theme(
            self, brandId, themeId, theme,
            keep_empty_params=False
    ):
        """
        Updates a theme for a brand
        Args:
            brand_id {str}
            theme_id {str}
            {theme}
        Returns:
            ThemeResponse
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes/{themeId}
            """)

        if isinstance(theme, dict):
            body = theme
        else:
            body = theme.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, ThemeResponse)

        if error:
            return (None, response, error)

        try:
            result = ThemeResponse(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_brand_theme_background_image(
            self, brandId, themeId,
            keep_empty_params=False
    ):
        """
        Deletes a Theme background image
        Args:
            brand_id {str}
            theme_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes/{themeId}
                /background-image
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def upload_brand_theme_background_image(
            self, brandId, themeId, file,
            keep_empty_params=False
    ):
        """
        Updates the background image for your Theme
        Args:
            brand_id {str}
            theme_id {str}
        Returns:
            ImageUploadResponse
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes/{themeId}
                /background-image
            """)

        body = {}
        headers = {}
        form = {
            "file": file,
        }

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, ImageUploadResponse)

        if error:
            return (None, response, error)

        try:
            result = ImageUploadResponse(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_brand_theme_favicon(
            self, brandId, themeId,
            keep_empty_params=False
    ):
        """
        Deletes a Theme favicon. The org then uses the Okta def
        ault favicon.
        Args:
            brand_id {str}
            theme_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes/{themeId}/favicon
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def upload_brand_theme_favicon(
            self, brandId, themeId, file,
            keep_empty_params=False
    ):
        """
        Updates the favicon for your theme
        Args:
            brand_id {str}
            theme_id {str}
        Returns:
            ImageUploadResponse
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes/{themeId}/favicon
            """)

        body = {}
        headers = {}
        form = {
            "file": file,
        }

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, ImageUploadResponse)

        if error:
            return (None, response, error)

        try:
            result = ImageUploadResponse(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_brand_theme_logo(
            self, brandId, themeId,
            keep_empty_params=False
    ):
        """
        Deletes a Theme logo. The org then uses the Okta defaul
        t logo.
        Args:
            brand_id {str}
            theme_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes/{themeId}/logo
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def upload_brand_theme_logo(
            self, brandId, themeId, file,
            keep_empty_params=False
    ):
        """
        Updates the logo for your Theme
        Args:
            brand_id {str}
            theme_id {str}
        Returns:
            ImageUploadResponse
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/brands/{brandId}/themes/{themeId}/logo
            """)

        body = {}
        headers = {}
        form = {
            "file": file,
        }

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, ImageUploadResponse)

        if error:
            return (None, response, error)

        try:
            result = ImageUploadResponse(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)



"""
Copyright 2020 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from urllib.parse import urlencode
from okta.models.sms_template\
    import SmsTemplate
from okta.utils import format_url
from okta.api_client import APIClient


class SmsTemplateClient(APIClient):
    """
    A Client object for the SmsTemplate resource.
    """

    def __init__(self):
        self._base_url = ""

    async def list_sms_templates(
            self, query_params={},
            keep_empty_params=False
    ):
        """
        Enumerates custom SMS templates in your organization. A
        subset of templates can be returned that match a templ
        ate type.
        Args:
            query_params {dict}: Map of query parameters for request
            [query_params.templateType] {str}
        Returns:
            list: Collection of SmsTemplate instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/templates/sms
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, SmsTemplate)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(SmsTemplate(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def create_sms_template(
            self, sms_template,
            keep_empty_params=False
    ):
        """
        Adds a new custom SMS template to your organization.
        Args:
            {sms_template}
        Returns:
            SmsTemplate
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/templates/sms
            """)

        if isinstance(sms_template, dict):
            body = sms_template
        else:
            body = sms_template.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, SmsTemplate)

        if error:
            return (None, response, error)

        try:
            result = SmsTemplate(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_sms_template(
            self, templateId,
            keep_empty_params=False
    ):
        """
        Removes an SMS template.
        Args:
            template_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/templates/sms/{templateId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def get_sms_template(
            self, templateId,
            keep_empty_params=False
    ):
        """
        Fetches a specific template by `id`
        Args:
            template_id {str}
        Returns:
            SmsTemplate
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/templates/sms/{templateId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, SmsTemplate)

        if error:
            return (None, response, error)

        try:
            result = SmsTemplate(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def partial_update_sms_template(
            self, templateId, sms_template,
            keep_empty_params=False
    ):
        """
        Updates only some of the SMS template properties:
        Args:
            template_id {str}
            {sms_template}
        Returns:
            SmsTemplate
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/templates/sms/{templateId}
            """)

        if isinstance(sms_template, dict):
            body = sms_template
        else:
            body = sms_template.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, SmsTemplate)

        if error:
            return (None, response, error)

        try:
            result = SmsTemplate(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_sms_template(
            self, templateId, sms_template,
            keep_empty_params=False
    ):
        """
        Updates the SMS template.
        Args:
            template_id {str}
            {sms_template}
        Returns:
            SmsTemplate
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/templates/sms/{templateId}
            """)

        if isinstance(sms_template, dict):
            body = sms_template
        else:
            body = sms_template.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, SmsTemplate)

        if error:
            return (None, response, error)

        try:
            result = SmsTemplate(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)



import pytest
from tests.mocks import MockOktaClient
import okta.models as models
from http import HTTPStatus
from okta.errors.okta_api_error import OktaAPIError
from okta.client import Client


class TestFactorsResource:
    """
    Integration Tests for the Factors Resource
    """
    SDK_PREFIX = "python_sdk"

    @pytest.mark.asyncio
    @pytest.mark.skip
    async def test_enroll_security_question_factor(self):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create User
        user_profile = models.UserProfile()
        user_profile.first_name = "John"
        user_profile.last_name = "Doe-Security-Question"
        user_profile.email = "John.Doe-Security-Question@example.com"
        user_profile.login = "John.Doe-Security-Question@example.com"

        create_user_req = models.CreateUserRequest({
            "credentials": models.UserCredentials({
                "password": models.PasswordCredential({
                    "value": "Password150kta"
                })
            }),
            "profile": user_profile
        })

        try:
            created_user, _, err = await client.create_user(create_user_req)
            assert err is None
            assert isinstance(created_user, models.User)

            # Create and add factor
            sec_q_factor = models.SecurityQuestionUserFactor({
                "factorType": models.FactorType.QUESTION,
                "provider": models.FactorProvider.OKTA,
                "profile": models.SecurityQuestionUserFactorProfile({
                    "question": "disliked_food",
                    "answer": "lasagna"
                })
            })

            enrolled_factor, _, err = await \
                client.enroll_factor(created_user.id, sec_q_factor)
            assert err is None
            assert isinstance(enrolled_factor, models.SecurityQuestionUserFactor)

            # List factor to validate
            users_factors, _, err = await client.list_factors(created_user.id)
            assert err is None
            assert len(users_factors) > 0 and len(users_factors) == 1
            assert isinstance(users_factors[0], models.SecurityQuestionUserFactor)
            assert users_factors[0].factor_type == models.FactorType.QUESTION
            assert users_factors[0].id == sec_q_factor.id
            assert users_factors[0].profile.question ==\
                sec_q_factor.profile.question
            assert users_factors[0].profile.answer == sec_q_factor.profile.answer
            assert users_factors[0].profile.question_text

        finally:
            errors = []
            # Deactivate + delete user
            try:
                _, err = await client.deactivate_user(created_user.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.deactivate_or_delete_user(created_user.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_enroll_sms_factor(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create User
        user_profile = models.UserProfile()
        user_profile.first_name = "John"
        user_profile.last_name = "Doe-SMS"
        user_profile.email = "John.Doe-SMS@example.com"
        user_profile.login = "John.Doe-SMS@example.com"

        create_user_req = models.CreateUserRequest({
            "credentials": models.UserCredentials({
                "password": models.PasswordCredential({
                    "value": "Password150kta"
                })
            }),
            "profile": user_profile
        })

        try:
            created_user, _, err = await client.create_user(create_user_req)
            assert err is None
            assert isinstance(created_user, models.User)

            # Create and add factor
            sms_factor = models.SmsUserFactor({
                "profile": models.SmsUserFactorProfile({
                    "phoneNumber": "+12345678901"
                })
            })

            enrolled_factor, _, err = await \
                client.enroll_factor(created_user.id, sms_factor)
            assert err is None
            assert isinstance(enrolled_factor, models.SmsUserFactor)

            # List factor to validate
            users_factors, _, err = await client.list_factors(created_user.id)
            assert err is None
            assert len(users_factors) > 0 and len(users_factors) == 1
            assert isinstance(users_factors[0], models.SmsUserFactor)
            assert users_factors[0].id == enrolled_factor.id
            assert users_factors[0].factor_type == models.FactorType.SMS
            assert users_factors[0].profile.phone_number ==\
                sms_factor.profile.phone_number

        finally:
            errors = []
            # Deactivate + delete user
            try:
                _, err = await client.deactivate_user(created_user.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.deactivate_or_delete_user(created_user.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_factors_new_user(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create User
        user_profile = models.UserProfile()
        user_profile.first_name = "John"
        user_profile.last_name = "Doe-List-Factor"
        user_profile.email = "John.Doe-List-Factor@example.com"
        user_profile.login = "John.Doe-List-Factor@example.com"

        create_user_req = models.CreateUserRequest({
            "credentials": models.UserCredentials({
                "password": models.PasswordCredential({
                    "value": "Password150kta"
                })
            }),
            "profile": user_profile
        })

        try:
            created_user, _, err = await client.create_user(create_user_req)
            assert err is None
            assert isinstance(created_user, models.User)

            # List factor to validate
            users_factors, _, err = await client.list_factors(created_user.id)
            assert err is None
            assert len(users_factors) == 0

        finally:
            errors = []
            # Deactivate + delete user
            try:
                _, err = await client.deactivate_user(created_user.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.deactivate_or_delete_user(created_user.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_get_factor(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create User
        user_profile = models.UserProfile()
        user_profile.first_name = "John"
        user_profile.last_name = "Doe-Get-Factor"
        user_profile.email = "John.Doe-Get-Factor@example.com"
        user_profile.login = "John.Doe-Get-Factor@example.com"

        create_user_req = models.CreateUserRequest({
            "credentials": models.UserCredentials({
                "password": models.PasswordCredential({
                    "value": "Password150kta"
                })
            }),
            "profile": user_profile
        })

        try:
            created_user, _, err = await client.create_user(create_user_req)
            assert err is None
            assert isinstance(created_user, models.User)

            # Create and add factor
            sms_factor = models.SmsUserFactor({
                "profile": models.SmsUserFactorProfile({
                    "phoneNumber": "+12345678901"
                })
            })

            enrolled_factor, _, err = await \
                client.enroll_factor(created_user.id, sms_factor)
            assert err is None
            assert isinstance(enrolled_factor, models.SmsUserFactor)

            # Get factor to validate
            retrieved_user_factor, _, err = await client.get_factor(
                created_user.id, enrolled_factor.id
            )
            assert err is None
            assert isinstance(retrieved_user_factor, models.SmsUserFactor)
            assert retrieved_user_factor.id == enrolled_factor.id
            assert retrieved_user_factor.factor_type == models.FactorType.SMS
            assert retrieved_user_factor.profile.phone_number ==\
                sms_factor.profile.phone_number

        finally:
            errors = []
            # Deactivate + delete user
            try:
                _, err = await client.deactivate_user(created_user.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.deactivate_or_delete_user(created_user.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_delete_factor(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create User
        user_profile = models.UserProfile()
        user_profile.first_name = "John"
        user_profile.last_name = "Doe-Delete-Factor"
        user_profile.email = "John.Doe-Delete-Factor@example.com"
        user_profile.login = "John.Doe-Delete-Factor@example.com"

        create_user_req = models.CreateUserRequest({
            "credentials": models.UserCredentials({
                "password": models.PasswordCredential({
                    "value": "Password150kta"
                })
            }),
            "profile": user_profile
        })

        try:
            created_user, _, err = await client.create_user(create_user_req)
            assert err is None
            assert isinstance(created_user, models.User)

            # Create and add factor
            sms_factor = models.SmsUserFactor({
                "profile": models.SmsUserFactorProfile({
                    "phoneNumber": "+12345678901"
                })
            })

            enrolled_factor, _, err = await \
                client.enroll_factor(created_user.id, sms_factor)
            assert err is None
            assert isinstance(enrolled_factor, models.SmsUserFactor)

            # Get factor to validate
            retrieved_user_factor, _, err = await client.get_factor(
                created_user.id, enrolled_factor.id
            )
            assert err is None
            assert isinstance(retrieved_user_factor, models.SmsUserFactor)
            assert retrieved_user_factor.id == enrolled_factor.id
            assert retrieved_user_factor.factor_type == models.FactorType.SMS
            assert retrieved_user_factor.profile.phone_number ==\
                sms_factor.profile.phone_number

            # Delete factor
            _, err = await client.delete_factor(created_user.id,
                                                enrolled_factor.id)
            assert err is None

            # Get factor to validate
            retrieved_user_factor, resp, err = await client.get_factor(
                created_user.id, enrolled_factor.id
            )
            assert err is not None
            assert isinstance(err, OktaAPIError)
            assert retrieved_user_factor is None
            assert resp.get_status() == HTTPStatus.NOT_FOUND

        finally:
            errors = []
            # Deactivate + delete user
            try:
                _, err = await client.deactivate_user(created_user.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.deactivate_or_delete_user(created_user.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.skip
    @pytest.mark.asyncio
    async def test_reset_factors(self):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create User
        user_profile = models.UserProfile()
        user_profile.first_name = "John"
        user_profile.last_name = "Doe-Reset-Factor"
        user_profile.email = "John.Doe-Reset-Factor@example.com"
        user_profile.login = "John.Doe-Reset-Factor@example.com"

        create_user_req = models.CreateUserRequest({
            "credentials": models.UserCredentials({
                "password": models.PasswordCredential({
                    "value": "Password150kta"
                })
            }),
            "profile": user_profile
        })

        try:
            created_user, _, err = await client.create_user(create_user_req)
            assert err is None
            assert isinstance(created_user, models.User)

            # Create and add factor
            sms_factor = models.SmsUserFactor({
                "profile": models.SmsUserFactorProfile({
                    "phoneNumber": "+12345678901"
                })
            })

            enrolled_factor, _, err = await \
                client.enroll_factor(created_user.id, sms_factor)
            assert err is None
            assert isinstance(enrolled_factor, models.SmsUserFactor)

            # Get factor to validate
            retrieved_user_factor, _, err = await client.get_factor(
                created_user.id, enrolled_factor.id
            )
            assert err is None
            assert isinstance(retrieved_user_factor, models.SmsUserFactor)
            assert retrieved_user_factor.id == enrolled_factor.id
            assert retrieved_user_factor.factor_type == models.FactorType.SMS
            assert retrieved_user_factor.profile.phone_number ==\
                sms_factor.profile.phone_number

            # Reset factor
            _, err = await client.reset_factors(created_user.id)
            assert err is None

            # List factors to validate
            users_factors, _, err = await client.list_factors(created_user.id)
            assert err is None
            assert len(users_factors) == 0

        finally:
            errors = []
            # Deactivate + delete user
            try:
                _, err = await client.deactivate_user(created_user.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.deactivate_or_delete_user(created_user.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.asyncio
    @pytest.mark.skip
    async def test_list_supported_security_questions(self):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Create User
        user_profile = models.UserProfile()
        user_profile.first_name = "John"
        user_profile.last_name = "Doe-List-Security-Question"
        user_profile.email = "John.Doe-List-Security-Question@example.com"
        user_profile.login = "John.Doe-List-Security-Question@example.com"

        create_user_req = models.CreateUserRequest({
            "credentials": models.UserCredentials({
                "password": models.PasswordCredential({
                    "value": "Password150kta"
                })
            }),
            "profile": user_profile
        })

        try:
            created_user, _, err = await client.create_user(create_user_req)
            assert err is None
            assert isinstance(created_user, models.User)

            # Create and add factor
            sec_q_factor = models.SecurityQuestionUserFactor({
                "factorType": models.FactorType.QUESTION,
                "provider": models.FactorProvider.OKTA,
                "profile": models.SecurityQuestionUserFactorProfile({
                    "question": "disliked_food",
                    "answer": "lasagna"
                })
            })

            enrolled_factor, _, err = await \
                client.enroll_factor(created_user.id, sec_q_factor)
            assert err is None
            assert isinstance(enrolled_factor, models.SecurityQuestionUserFactor)

            # List factor to validate
            users_factors, _, err = await client.list_factors(created_user.id)
            assert err is None
            assert len(users_factors) > 0 and len(users_factors) == 1
            assert isinstance(users_factors[0], models.SecurityQuestionUserFactor)
            assert users_factors[0].factor_type == models.FactorType.QUESTION
            assert users_factors[0].id == sec_q_factor.id
            assert users_factors[0].profile.question ==\
                sec_q_factor.profile.question
            assert users_factors[0].profile.answer == sec_q_factor.profile.answer
            assert users_factors[0].profile.question_text

            # List Security q's
            retrieved_questions, _, err = await \
                client.list_supported_security_questions(created_user.id)
            assert err is None
            assert next((question for question in retrieved_questions
                         if question.profile.question ==
                         enrolled_factor.profile.question))

        finally:
            errors = []
            # Deactivate + delete user
            try:
                _, err = await client.deactivate_user(created_user.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.deactivate_or_delete_user(created_user.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0



"""
Copyright 2020 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from urllib.parse import urlencode
from okta.models.log_event\
    import LogEvent
from okta.utils import format_url
from okta.api_client import APIClient


class LogEventClient(APIClient):
    """
    A Client object for the LogEvent resource.
    """

    def __init__(self):
        self._base_url = ""

    async def get_logs(
            self, query_params={},
            keep_empty_params=False
    ):
        """
        The Okta System Log API provides read access to your or
        ganization’s system log. This API provides more functio
        nality than the Events API
        Args:
            query_params {dict}: Map of query parameters for request
            [query_params.since] {str}
            [query_params.until] {str}
            [query_params.filter] {str}
            [query_params.q] {str}
            [query_params.limit] {str}
            [query_params.sortOrder] {str}
            [query_params.after] {str}
        Returns:
            list: Collection of LogEvent instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/logs
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, LogEvent)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(LogEvent(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)



"""
Copyright 2020 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from okta.models.domain_list_response\
    import DomainListResponse
from okta.models.domain\
    import Domain
from okta.utils import format_url
from okta.api_client import APIClient


class DomainClient(APIClient):
    """
    A Client object for the Domain resource.
    """

    def __init__(self):
        self._base_url = ""

    async def list_domains(
            self,
            keep_empty_params=False
    ):
        """
        List all verified custom Domains for the org.
        Args:
        Returns:
            DomainListResponse
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/domains
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, DomainListResponse)

        if error:
            return (None, response, error)

        try:
            result = DomainListResponse(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def create_domain(
            self, domain,
            keep_empty_params=False
    ):
        """
        Creates your domain.
        Args:
            {domain}
        Returns:
            Domain
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/domains
            """)

        if isinstance(domain, dict):
            body = domain
        else:
            body = domain.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Domain)

        if error:
            return (None, response, error)

        try:
            result = Domain(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_domain(
            self, domainId,
            keep_empty_params=False
    ):
        """
        Deletes a Domain by `id`.
        Args:
            domain_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/domains/{domainId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def get_domain(
            self, domainId,
            keep_empty_params=False
    ):
        """
        Fetches a Domain by `id`.
        Args:
            domain_id {str}
        Returns:
            Domain
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/domains/{domainId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Domain)

        if error:
            return (None, response, error)

        try:
            result = Domain(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def create_certificate(
            self, domainId, domain_certificate,
            keep_empty_params=False
    ):
        """
        Creates the Certificate for the Domain.
        Args:
            domain_id {str}
            {domain_certificate}
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/domains/{domainId}/certificate
            """)

        if isinstance(domain_certificate, dict):
            body = domain_certificate
        else:
            body = domain_certificate.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def verify_domain(
            self, domainId,
            keep_empty_params=False
    ):
        """
        Verifies the Domain by `id`.
        Args:
            domain_id {str}
        Returns:
            Domain
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/domains/{domainId}/verify
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Domain)

        if error:
            return (None, response, error)

        try:
            result = Domain(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)



import pytest
from tests.mocks import MockOktaClient
import okta.models as models
from http import HTTPStatus
from okta.errors.okta_api_error import OktaAPIError


class TestUserSchemaResource:
    """
    Integration Tests for the User Schema Resource
    """
    SDK_PREFIX = "python_sdk"

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_get_user_schema(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        # Get User Schema
        user_schema, resp, err = await client.get_user_schema('default')
        assert err is None

        # Validate schema attributes
        assert user_schema.definitions.base.id == '#base'
        assert user_schema.definitions.base.type == 'object'
        assert 'login' in user_schema.definitions.base.required
        assert user_schema.definitions.base.properties.login.title == 'Username'
        assert user_schema.definitions.base.properties.login.type == 'string'
        assert user_schema.definitions.base.properties.login.required is True
        assert user_schema.definitions.base.properties.login.mutability == 'READ_WRITE'
        assert user_schema.definitions.base.properties.login.scope == 'NONE'
        assert user_schema.definitions.base.properties.login.min_length == 5
        assert user_schema.definitions.base.properties.login.max_length == 100
        assert user_schema.definitions.base.properties.login.permissions[0].principal == 'SELF'
        assert user_schema.definitions.base.properties.login.permissions[0].action == 'READ_ONLY'

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_update_user_profile(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        try:
            # Get User Schema
            user_schema, _, err = await client.get_user_schema("default")
            assert err is None

            # Add custom attribute
            attr_config = {
                'title': 'Test Custom User Attribute',
                'type': 'string',
                'description': 'test custom user attribute',
                'minLength': 1,
                'maxLength': 20,
                'permissions': [models.UserSchemaAttributePermission({
                                   'action': 'READ_WRITE',
                                   'principal': 'SELF'
                               })]
            }

            custom_attr = models.UserSchemaAttribute(attr_config)
            user_schema.definitions.custom.properties = {'testCustomAttribute': custom_attr.as_dict()}

            # Update user schema
            updated_user_schema, _, err = await client.update_user_profile('default', user_schema)
            assert err is None

            # Check custom attribute
            assert updated_user_schema.definitions.custom.properties['testCustomAttribute']['title'] == 'Test Custom User Attribute'
        finally:
            errors = []
            try:
                # Remove custom attribute
                updated_user_schema.definitions.custom.properties['testCustomAttribute'] = None
                # Update user schema
                updated_user_schema, _, err = await client.update_user_profile('default', updated_user_schema)
                assert err is None
            except Exception as exc:
                errors.append(exc)

            assert len(errors) == 0



import aiohttp
import asyncio
from okta.client import Client as OktaClient
from okta.models import DnsRecord, DomainCertificate, DomainCertificateMetadata, Domain


CREATE_DOMAIN_RESP = """{
    "id": "OcDz6iRyjkaCTXkdo0g3",
    "domain": "login.example.com",
    "validationStatus": "NOT_STARTED",
    "dnsRecords": [
        {
            "fqdn": "_oktaverification.login.example.com",
            "values": [
                "79496f234c814638b1cc44f51a782781"
            ],
            "recordType": "TXT"
        },
        {
            "fqdn": "login.sigmanetcorp.us",
            "values": [
                "test.customdomains.okta1.com"
            ],
            "recordType": "CNAME"
        }
    ],
    "_links": {
        "self": {
            "href": "https://test.okta.com/api/v1/domains/OcDz6iRyjkaCTXkdo0g3",
            "hints": {
                "allow": [
                    "GET",
                    "DELETE"
                ]
            }
        },
        "verify": {
            "href": "https://test.okta.com/api/v1/domains/OcDz6iRyjkaCTXkdo0g3/verify",
            "hints": {
                "allow": [
                    "POST"
                ]
            }
        }
    }
}"""


VERIFY_DOMAIN_RESP = """{
    "id": "OcDz6iRyjkaCTXkdo0g3",
    "domain": "login.example.com",
    "certificateSourceType": "MANUAL",
    "validationStatus": "VERIFIED",
    "dnsRecords": [
        {
            "fqdn": "_oktaverification.login.example.com",
            "values": [
                "79496f234c814638b1cc44f51a782781"
            ],
            "recordType": "TXT"
        },
        {
            "fqdn": "login.sigmanetcorp.us",
            "values": [
                "test.customdomains.okta1.com"
            ],
            "recordType": "CNAME"
        }
    ],
    "_links": {
           "certificate": {
               "href": "https://test.okta.com/api/v1/domains/OcDz6iRyjkaCTXkdo0g3/certificate",
               "hints": {
                   "allow": [
                       "PUT"
                   ]
               }
           },
        "self": {
            "href": "https://test.okta.com/api/v1/domains/OcDz6iRyjkaCTXkdo0g3",
            "hints": {
                "allow": [
                    "GET",
                    "DELETE"
                ]
            }
        }
    }
}"""


GET_DOMAIN_RESP = """{
    "id": "OcDz6iRyjkaCTXkdo0g3",
    "domain": "login.example.com",
    "certificateSourceType": "MANUAL",
    "validationStatus": "COMPLETED",
    "dnsRecords": [
        {
            "fqdn": "_oktaverification.login.example.com",
            "values": [
                "79496f234c814638b1cc44f51a782781"
            ],
            "recordType": "TXT"
        },
        {
            "fqdn": "login.example.com",
            "values": [
                "test.okta.com.customdomains.okta1.com"
            ],
            "recordType": "CNAME"
        }
    ],
    "publicCertificate": {
        "subject": "CN=login.example.com",
        "fingerprint": "73:68:82:7B:83:2E:48:29:A5:5E:E8:40:41:80:B3:AA:03:C4:42:43:05:73:45:BC:AA:47:00:23:A3:70:E5:C4",
        "expiration": "2021-05-11T05:13:05.000Z"
    },
    "_links": {
        "certificate": {
            "href": "https://test.okta.com/api/v1/domains/OcDz6iRyjkaCTXkdo0g3/certificate",
            "hints": {
                "allow": [
                    "PUT"
                ]
            }
        },
        "self": {
            "href": "https://test.okta.com/api/v1/domains/OcDz6iRyjkaCTXkdo0g3",
            "hints": {
                "allow": [
                    "DELETE"
                ]
            }
        },
        "verify": {
            "href": "https://test.okta.com/api/v1/domains/OcDz6iRyjkaCTXkdo0g3/verify",
            "hints": {
                "allow": [
                    "POST"
                ]
            }
        }
    }
}"""


class TestDomainResource:
    """
    Unit Tests for the Domain Resource
    """

    def test_get_domain(self, monkeypatch):
        org_url = "https://test.okta.com"
        token = "TOKEN"
        config = {'orgUrl': org_url, 'token': token}
        client = OktaClient(config)

        # mock http requests
        class MockHTTPRequest():

            _mocked_response = None

            def __call__(self, **params):
                self.request_info = params
                self.headers = params['headers']
                self.url = params['url']
                self.content_type = 'application/json'
                self.links = ''
                self.text = MockHTTPRequest.mock_response_text
                self.status = 200
                return self

            async def __aenter__(self):
                return self

            async def __aexit__(self, exc_type, exc, tb):
                pass

            @staticmethod
            async def mock_response_text():
                return GET_DOMAIN_RESP

        mock_http_request = MockHTTPRequest()
        monkeypatch.setattr(aiohttp.ClientSession, 'request', mock_http_request)

        domain_resp, _, err = asyncio.run(client.get_domain('OcDz6iRyjkaCTXkdo0g3'))
        assert err is None
        assert isinstance(domain_resp, Domain)
        assert isinstance(domain_resp.public_certificate, DomainCertificateMetadata)
        assert len(domain_resp.dns_records) > 0
        for dns_record in domain_resp.dns_records:
            assert isinstance(dns_record, DnsRecord)
        assert domain_resp.domain == 'login.example.com'

    def test_create_certificate_and_verify_domain(self, monkeypatch):
        org_url = "https://test.okta.com"
        token = "TOKEN"
        config = {'orgUrl': org_url, 'token': token}
        client = OktaClient(config)

        # mock http requests
        class MockHTTPRequest():

            _mocked_response = None

            def __call__(self, **params):
                self.request_info = params
                self.headers = params['headers']
                self.url = params['url']
                self.content_type = 'application/json'
                self.links = ''
                MockHTTPRequest._set_mocked_response(params)
                self.text = MockHTTPRequest.mock_response_text
                self.status = 200
                return self

            async def __aenter__(self):
                return self

            async def __aexit__(self, exc_type, exc, tb):
                pass

            @staticmethod
            def _set_mocked_response(params):
                if params['method'] == 'POST' and params['url'] == 'https://test.okta.com/api/v1/domains':
                    MockHTTPRequest._mocked_response = CREATE_DOMAIN_RESP
                elif params['method'] == 'PUT' and params['url'] == 'https://test.okta.com/api/v1/domains/OcDz6iRyjkaCTXkdo0g3/certificate':
                    MockHTTPRequest._mocked_response = '{}'
                elif params['method'] == 'POST' and params['url'] == 'https://test.okta.com/api/v1/domains/OcDz6iRyjkaCTXkdo0g3/verify':
                    MockHTTPRequest._mocked_response = VERIFY_DOMAIN_RESP

            @staticmethod
            async def mock_response_text():
                return MockHTTPRequest._mocked_response

        mock_http_request = MockHTTPRequest()
        monkeypatch.setattr(aiohttp.ClientSession, 'request', mock_http_request)

        domain_config = {
            "domain": "login.example.com",
            "certificateSourceType": "MANUAL"
        }
        domain_resp, _, err = asyncio.run(client.create_domain(domain_config))
        assert err is None

        cert = DomainCertificate({'type': 'PEM',
                                  'certificate': 'test_certificate',
                                  'privateKey': 'test_private_key'})
        cert_resp, err = asyncio.run(client.create_certificate(domain_resp.id, cert))
        assert err is None

        domain_resp, _, err = asyncio.run(client.verify_domain(domain_resp.id))
        assert err is None
        assert domain_resp.validation_status == 'VERIFIED'



import tests.mocks as mocks
import pytest
from okta.client import Client
from okta.request_executor import RequestExecutor

REQUEST_TIMEOUT = mocks.REQUEST_TIMEOUT
ORG_URL = mocks.ORG_URL
API_TOKEN = mocks.API_TOKEN
CLIENT_ID = mocks.CLIENT_ID
SCOPES = mocks.SCOPES
PRIVATE_KEY = mocks.PRIVATE_KEY
GET_USERS_CALL = mocks.GET_USERS_CALL
GET_OAUTH_CLIENTS_CALL = mocks.GET_OAUTH_CLIENTS_CALL
CLIENT_CONFIG = mocks.CLIENT_CONFIG
API_LIMIT = "?limit=1"


@ pytest.mark.asyncio
async def test_response_pagination_with_next(monkeypatch):
    ssws_client = Client({
        "orgUrl": ORG_URL,
        "token": API_TOKEN
    })

    req, error = await ssws_client.get_request_executor()\
        .create_request("GET",
                        GET_USERS_CALL + API_LIMIT,
                        {},
                        {})

    monkeypatch.setattr(RequestExecutor, 'fire_request',
                        mocks.mock_GET_HTTP_Client_response_valid_with_next)

    result, error = await ssws_client.get_request_executor().execute(req)

    assert result.get_body() is not None
    assert result.has_next()
    assert result._next.startswith(GET_USERS_CALL)
    monkeypatch.setattr(RequestExecutor, 'fire_request',
                        mocks.mock_GET_HTTP_Client_response_valid)
    assert await result.next() is not None
    assert not result.has_next()
    with pytest.raises(StopAsyncIteration):
        await result.next()


@ pytest.mark.asyncio
async def test_response_pagination_with_next_not_starting_with_api(monkeypatch):
    ssws_client = Client(CLIENT_CONFIG)

    req, error = await ssws_client.get_request_executor() \
        .create_request("GET",
                        GET_OAUTH_CLIENTS_CALL + API_LIMIT,
                        {},
                        {})

    monkeypatch.setattr(RequestExecutor, 'fire_request',
                        mocks.mock_GET_HTTP_Client_response_valid_with_next)

    result, error = await ssws_client.get_request_executor().execute(req)

    assert result.get_body() is not None
    assert result.has_next()
    assert result._next.startswith(GET_OAUTH_CLIENTS_CALL)
    monkeypatch.setattr(RequestExecutor, 'fire_request',
                        mocks.mock_GET_HTTP_Client_response_valid)
    assert await result.next() is not None
    assert not result.has_next()
    with pytest.raises(StopAsyncIteration):
        await result.next()


@ pytest.mark.asyncio
async def test_response_pagination_with_no_next(monkeypatch):
    ssws_client = Client({
        "orgUrl": ORG_URL,
        "token": API_TOKEN
    })

    req, error = await ssws_client.get_request_executor()\
        .create_request("GET",
                        GET_USERS_CALL + API_LIMIT,
                        {},
                        {})

    monkeypatch.setattr(RequestExecutor, 'fire_request',
                        mocks.mock_GET_HTTP_Client_response_valid)

    result, error = await ssws_client.get_request_executor().execute(req)

    assert result.get_body() is not None
    assert not result.has_next()
    with pytest.raises(StopAsyncIteration):
        await result.next()


@ pytest.mark.asyncio
async def test_response_headers(monkeypatch):
    ssws_client = Client({
        "orgUrl": ORG_URL,
        "token": API_TOKEN
    })

    req, error = await ssws_client.get_request_executor()\
        .create_request("GET",
                        GET_USERS_CALL + API_LIMIT,
                        {},
                        {})

    monkeypatch.setattr(RequestExecutor, 'fire_request',
                        mocks.mock_GET_HTTP_Client_response_valid)

    result, error = await ssws_client.get_request_executor().execute(req)
    assert result.get_body() is not None
    assert result.get_headers() == mocks.MockHTTPResponseDetails().headers



from setuptools import setup, find_packages
import os


def get_version():
    # Get version number from VERSION file
    with open(os.path.join(os.path.dirname(__file__), "okta", "__init__.py")) \
            as version_file:
        # File has format: __version__ = '{version_number}'
        line = version_file.read().split("=")
        version_number = line[1].strip().replace("'", "")
        return version_number


setup(
    name="okta",
    version=get_version(),
    author="Okta, Inc.",
    author_email="devex@okta.com",
    url="https://github.com/okta/okta-sdk-python",
    license="Apache License 2.0",
    description="Python SDK for the Okta Management API",
    long_description=open("LONG_DESCRIPTION.md").read(),
    test_suite="tests",
    packages=find_packages(exclude=("tests",)),
    python_requires=">=3.6",
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "License :: OSI Approved :: Apache Software License",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
    install_requires=[
        "aiohttp",
        "flatdict",
        "pyyaml",
        "xmltodict",
        "yarl",
        "pycryptodome",
        "python-jose",
        "aenum",
        "pydash"
    ]
)



import pytest
from tests.mocks import MockOktaClient
from okta.models import (
                            NetworkZone,
                            NetworkZoneType,
                            NetworkZoneStatus,
                            NetworkZoneUsage,
                            NetworkZoneAddress,
                            NetworkZoneAddressType
                        )


class TestNetworkZoneResource:
    """
    Integration Tests for the NetworkZone Resource
    """

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_network_zones(self, fs):
        client = MockOktaClient(fs)
        resp, _, err = await client.list_network_zones()
        for network_zone in resp:
            assert isinstance(network_zone, NetworkZone)
            assert isinstance(network_zone.type, NetworkZoneType)
            assert isinstance(network_zone.status, NetworkZoneStatus)
            assert isinstance(network_zone.usage, NetworkZoneUsage)

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_create_and_delete_network_zone(self, fs):
        client = MockOktaClient(fs)
        network_zone_config = {
            "type": "IP",
            "id": None,
            "name": "testNetworkZone",
            "status": "ACTIVE",
            "created": None,
            "lastUpdated": None,
            "gateways": [
              {
                "type": "CIDR",
                "value": "1.2.3.4/24"
              }
            ],
            "proxies": [
              {
                "type": "CIDR",
                "value": "2.2.3.4/24"
              }
            ]
        }
        resp, _, err = await client.create_network_zone(network_zone_config)
        assert err is None
        try:
            assert resp.name == 'testNetworkZone'
            assert isinstance(resp.gateways[0], NetworkZoneAddress)
            assert resp.gateways[0].value == '1.2.3.4/24'
            assert isinstance(resp.proxies[0], NetworkZoneAddress)
            assert resp.proxies[0].value == '2.2.3.4/24'
        finally:
            errors = []
            try:
                _, _, err = await client.deactivate_network_zone(resp.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_network_zone(resp.id)
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_update_network_zone(self, fs):
        client = MockOktaClient(fs)
        network_zone_config = {
            "type": "IP",
            "id": None,
            "name": "testNetworkZone",
            "status": "ACTIVE",
            "created": None,
            "lastUpdated": None,
            "gateways": [
              {
                "type": "CIDR",
                "value": "1.2.3.4/24"
              }
            ],
            "proxies": [
              {
                "type": "CIDR",
                "value": "2.2.3.4/24"
              }
            ]
        }
        resp, _, err = await client.create_network_zone(network_zone_config)
        assert err is None
        try:
            new_gateway = NetworkZoneAddress({'type': NetworkZoneAddressType('CIDR'),
                                              'value': '2.3.4.5/24'})
            resp.gateways.append(new_gateway)
            resp, _, err = await client.update_network_zone(resp.id, resp)
            assert len(resp.gateways) == 2
            assert resp.gateways[1].value == '2.3.4.5/24'
        finally:
            errors = []
            try:
                _, _, err = await client.deactivate_network_zone(resp.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            try:
                _, err = await client.delete_network_zone(resp.id)
                assert err is None
            except Exception as exc:
                errors.append(exc)
            assert len(errors) == 0



"""
Copyright 2020 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from urllib.parse import urlencode
from okta.models.group\
    import Group
from okta.models.group_rule\
    import GroupRule
from okta.models.application\
    import Application
from okta.models.role\
    import Role
from okta.models.catalog_application\
    import CatalogApplication
from okta.models.user\
    import User
from okta.utils import format_url
from okta.api_client import APIClient
from okta.constants import find_app_model


class GroupClient(APIClient):
    """
    A Client object for the Group resource.
    """

    def __init__(self):
        self._base_url = ""

    async def list_groups(
            self, query_params={},
            keep_empty_params=False
    ):
        """
        Enumerates groups in your organization with pagination.
        A subset of groups can be returned that match a suppor
        ted filter expression or query.
        Args:
            query_params {dict}: Map of query parameters for request
            [query_params.q] {str}
            [query_params.search] {str}
            [query_params.after] {str}
            [query_params.limit] {str}
            [query_params.expand] {str}
        Returns:
            list: Collection of Group instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Group)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(Group(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def create_group(
            self, group,
            keep_empty_params=False
    ):
        """
        Adds a new group with `OKTA_GROUP` type to your organiz
        ation.
        Args:
            {group}
        Returns:
            Group
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups
            """)

        if isinstance(group, dict):
            body = group
        else:
            body = group.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Group)

        if error:
            return (None, response, error)

        try:
            result = Group(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def list_group_rules(
            self, query_params={},
            keep_empty_params=False
    ):
        """
        Lists all group rules for your organization.
        Args:
            query_params {dict}: Map of query parameters for request
            [query_params.limit] {str}
            [query_params.after] {str}
            [query_params.search] {str}
            [query_params.expand] {str}
        Returns:
            list: Collection of GroupRule instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/rules
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, GroupRule)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(GroupRule(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def create_group_rule(
            self, group_rule,
            keep_empty_params=False
    ):
        """
        Creates a group rule to dynamically add users to the sp
        ecified group if they match the condition
        Args:
            {group_rule}
        Returns:
            GroupRule
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/rules
            """)

        if isinstance(group_rule, dict):
            body = group_rule
        else:
            body = group_rule.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, GroupRule)

        if error:
            return (None, response, error)

        try:
            result = GroupRule(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def delete_group_rule(
            self, ruleId, query_params={},
            keep_empty_params=False
    ):
        """
        Removes a specific group rule by id from your organizat
        ion
        Args:
            rule_id {str}
            query_params {dict}: Map of query parameters for request
            [query_params.removeUsers] {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/rules/{ruleId}
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def get_group_rule(
            self, ruleId, query_params={},
            keep_empty_params=False
    ):
        """
        Fetches a specific group rule by id from your organizat
        ion
        Args:
            rule_id {str}
            query_params {dict}: Map of query parameters for request
            [query_params.expand] {str}
        Returns:
            GroupRule
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/rules/{ruleId}
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, GroupRule)

        if error:
            return (None, response, error)

        try:
            result = GroupRule(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_group_rule(
            self, ruleId, group_rule,
            keep_empty_params=False
    ):
        """
        Updates a group rule. Only `INACTIVE` rules can be upda
        ted.
        Args:
            rule_id {str}
            {group_rule}
        Returns:
            GroupRule
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/rules/{ruleId}
            """)

        if isinstance(group_rule, dict):
            body = group_rule
        else:
            body = group_rule.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, GroupRule)

        if error:
            return (None, response, error)

        try:
            result = GroupRule(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def activate_group_rule(
            self, ruleId,
            keep_empty_params=False
    ):
        """
        Activates a specific group rule by id from your organiz
        ation
        Args:
            rule_id {str}
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/rules/{ruleId}/lifecycle/activate
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def deactivate_group_rule(
            self, ruleId,
            keep_empty_params=False
    ):
        """
        Deactivates a specific group rule by id from your organ
        ization
        Args:
            rule_id {str}
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/rules/{ruleId}/lifecycle/deactivate
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def delete_group(
            self, groupId,
            keep_empty_params=False
    ):
        """
        Removes a group with `OKTA_GROUP` type from your organi
        zation.
        Args:
            group_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def get_group(
            self, groupId,
            keep_empty_params=False
    ):
        """
        Fetches a group from your organization.
        Args:
            group_id {str}
        Returns:
            Group
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Group)

        if error:
            return (None, response, error)

        try:
            result = Group(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_group(
            self, groupId, group,
            keep_empty_params=False
    ):
        """
        Updates the profile for a group with `OKTA_GROUP` type
        from your organization.
        Args:
            group_id {str}
            {group}
        Returns:
            Group
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}
            """)

        if isinstance(group, dict):
            body = group
        else:
            body = group.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Group)

        if error:
            return (None, response, error)

        try:
            result = Group(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def list_assigned_applications_for_group(
            self, groupId, query_params={},
            keep_empty_params=False
    ):
        """
        Enumerates all applications that are assigned to a grou
        p.
        Args:
            group_id {str}
            query_params {dict}: Map of query parameters for request
            [query_params.after] {str}
            [query_params.limit] {str}
        Returns:
            list: Collection of Application instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}/apps
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Application)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(
                    find_app_model(item["signOnMode"], item["name"])(
                        self.form_response_body(item)
                        )
                    )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def list_group_assigned_roles(
            self, groupId, query_params={},
            keep_empty_params=False
    ):
        """
        Args:
            group_id {str}
            query_params {dict}: Map of query parameters for request
            [query_params.expand] {str}
        Returns:
            list: Collection of Role instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}/roles
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Role)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(Role(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def assign_role_to_group(
            self, groupId, assign_role_request, query_params={},
            keep_empty_params=False
    ):
        """
        Assigns a Role to a Group
        Args:
            group_id {str}
            {assign_role_request}
            query_params {dict}: Map of query parameters for request
            [query_params.disableNotifications] {str}
        Returns:
            Role
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}/roles
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        if isinstance(assign_role_request, dict):
            body = assign_role_request
        else:
            body = assign_role_request.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Role)

        if error:
            return (None, response, error)

        try:
            result = Role(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def remove_role_from_group(
            self, groupId, roleId,
            keep_empty_params=False
    ):
        """
        Unassigns a Role from a Group
        Args:
            group_id {str}
            role_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}/roles/{roleId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def get_role(
            self, groupId, roleId,
            keep_empty_params=False
    ):
        """
        Args:
            group_id {str}
            role_id {str}
        Returns:
            Role
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}/roles/{roleId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Role)

        if error:
            return (None, response, error)

        try:
            result = Role(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def list_app_targets_for_application_admin_role_for_group(
            self, groupId, roleId, query_params={},
            keep_empty_params=False
    ):
        """
        Lists all App targets for an `APP_ADMIN` Role assigned
        to a Group. This methods return list may include full A
        pplications or Instances. The response for an instance
        will have an `ID` value, while Application will not hav
        e an ID.
        Args:
            group_id {str}
            role_id {str}
            query_params {dict}: Map of query parameters for request
            [query_params.after] {str}
            [query_params.limit] {str}
        Returns:
            list: Collection of CatalogApplication instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}/roles/{roleId}/targets
                /catalog/apps
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, CatalogApplication)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(CatalogApplication(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def remove_app_target_from_application_admin_role_given_to_group(
            self, groupId, roleId, appName,
            keep_empty_params=False
    ):
        """
        Args:
            group_id {str}
            role_id {str}
            app_name {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}/roles/{roleId}/targets
                /catalog/apps/{appName}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def add_application_target_to_admin_role_given_to_group(
            self, groupId, roleId, appName,
            keep_empty_params=False
    ):
        """
        Args:
            group_id {str}
            role_id {str}
            app_name {str}
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}/roles/{roleId}/targets
                /catalog/apps/{appName}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def remove_app_target_from_admin_role_given_to_group(
            self, groupId, roleId, appName, applicationId,
            keep_empty_params=False
    ):
        """
        Remove App Instance Target to App Administrator Role gi
        ven to a Group
        Args:
            group_id {str}
            role_id {str}
            app_name {str}
            application_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}/roles/{roleId}/targets
                /catalog/apps/{appName}/{applicationId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def add_app_instance_target_to_app_admin_role_given_to_group(
            self, groupId, roleId, appName, applicationId,
            keep_empty_params=False
    ):
        """
        Add App Instance Target to App Administrator Role given
        to a Group
        Args:
            group_id {str}
            role_id {str}
            app_name {str}
            application_id {str}
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}/roles/{roleId}/targets
                /catalog/apps/{appName}/{applicationId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def list_group_targets_for_group_role(
            self, groupId, roleId, query_params={},
            keep_empty_params=False
    ):
        """
        Args:
            group_id {str}
            role_id {str}
            query_params {dict}: Map of query parameters for request
            [query_params.after] {str}
            [query_params.limit] {str}
        Returns:
            list: Collection of Group instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}/roles/{roleId}/targets
                /groups
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Group)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(Group(
                    self.form_response_body(item)
                    ))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def remove_group_target_from_group_admin_role_given_to_group(
            self, groupId, roleId, targetGroupId,
            keep_empty_params=False
    ):
        """
        Method for
        /api/v1/groups/{groupId}/roles/{roleId}/targets/groups/
        {targetGroupId}
        Args:
            group_id {str}
            role_id {str}
            target_group_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}/roles/{roleId}/targets
                /groups/{targetGroupId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def add_group_target_to_group_administrator_role_for_group(
            self, groupId, roleId, targetGroupId,
            keep_empty_params=False
    ):
        """
        Method for
        /api/v1/groups/{groupId}/roles/{roleId}/targets/groups/
        {targetGroupId}
        Args:
            group_id {str}
            role_id {str}
            target_group_id {str}
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}/roles/{roleId}/targets
                /groups/{targetGroupId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def list_group_users(
            self, groupId, query_params={},
            keep_empty_params=False
    ):
        """
        Enumerates all users that are a member of a group.
        Args:
            group_id {str}
            query_params {dict}: Map of query parameters for request
            [query_params.after] {str}
            [query_params.limit] {str}
        Returns:
            list: Collection of User instances.
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}/users
            """)
        if query_params:
            encoded_query_params = urlencode(query_params)
            api_url += f"/?{encoded_query_params}"

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, User)

        if error:
            return (None, response, error)

        try:
            result = []
            for item in response.get_body():
                result.append(User(item))
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def remove_user_from_group(
            self, groupId, userId,
            keep_empty_params=False
    ):
        """
        Removes a user from a group with 'OKTA_GROUP' type.
        Args:
            group_id {str}
            user_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}/users/{userId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def add_user_to_group(
            self, groupId, userId,
            keep_empty_params=False
    ):
        """
        Adds a user to a group with 'OKTA_GROUP' type.
        Args:
            group_id {str}
            user_id {str}
        """
        http_method = "put".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/groups/{groupId}/users/{userId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)



import copy
import pytest

from okta import models
from tests.mocks import MockOktaClient


class TestBrandsResource:
    """
    Integration Tests for the Brand and Themes Resource
    """

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_brands(self, fs):
        client = MockOktaClient(fs)
        brands, _, err = await client.list_brands()
        assert err is None
        assert len(brands) > 0
        for brand in brands:
            assert isinstance(brand, models.Brand)

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_get_brand(self, fs):
        client = MockOktaClient(fs)
        brands, _, err = await client.list_brands()
        assert err is None
        brand_id = brands[0].id
        brand, _, err = await client.get_brand(brand_id)
        assert err is None
        assert isinstance(brand, models.Brand)
        assert brand.id == brand_id

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_update_brand(self, fs):
        client = MockOktaClient(fs)
        custom_privacy_policy_url = 'https://www.someHost.com/privacy-policy'

        brands, _, err = await client.list_brands()
        assert err is None
        brand_id = brands[0].id
        brand, _, err = await client.get_brand(brand_id)
        assert err is None
        assert brand.id == brand_id
        assert brand.custom_privacy_policy_url != custom_privacy_policy_url

        new_brand = copy.deepcopy(brand)
        new_brand.agree_to_custom_privacy_policy = True
        new_brand.custom_privacy_policy_url = 'https://www.someHost.com/privacy-policy'

        try:
            updated_brand, _, err = await client.update_brand(brand_id, new_brand)
            assert err is None
            assert isinstance(updated_brand, models.Brand)
            assert brand.id == updated_brand.id
            assert updated_brand.custom_privacy_policy_url == custom_privacy_policy_url
        finally:
            # restore previous brand settings
            _, _, err = await client.update_brand(brand_id, brand)
            assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_brand_themes(self, fs):
        client = MockOktaClient(fs)
        brands, _, err = await client.list_brands()
        assert err is None
        brand_id = brands[0].id
        themes, _, err = await client.list_brand_themes(brand_id)
        assert err is None
        assert len(themes) > 0
        for theme in themes:
            assert isinstance(theme, models.ThemeResponse)

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_get_brand_theme(self, fs):
        client = MockOktaClient(fs)
        brands, _, err = await client.list_brands()
        assert err is None
        brand_id = brands[0].id
        themes, _, err = await client.list_brand_themes(brand_id)
        assert err is None
        theme_id = themes[0].id
        theme, _, err = await client.get_brand_theme(brand_id, theme_id)
        assert err is None
        assert isinstance(theme, models.ThemeResponse)
        assert isinstance(theme.email_template_touch_point_variant,
                          models.EmailTemplateTouchPointVariant)
        assert isinstance(theme.end_user_dashboard_touch_point_variant,
                          models.EndUserDashboardTouchPointVariant)
        assert isinstance(theme.error_page_touch_point_variant,
                          models.ErrorPageTouchPointVariant)
        assert isinstance(theme.sign_in_page_touch_point_variant,
                          models.SignInPageTouchPointVariant)

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_update_brand_theme(self, fs):
        client = MockOktaClient(fs)
        brands, _, err = await client.list_brands()
        assert err is None
        brand_id = brands[0].id
        themes, _, err = await client.list_brand_themes(brand_id)
        assert err is None
        theme_id = themes[0].id
        theme, _, err = await client.get_brand_theme(brand_id, theme_id)
        assert err is None

        update_theme_params = {'primaryColorHex': '#ababab',
                               'secondaryColorHex': '#ebebeb',
                               'emailTemplateTouchPointVariant': models.EmailTemplateTouchPointVariant('OKTA_DEFAULT'),
                               'endUserDashboardTouchPointVariant': models.EndUserDashboardTouchPointVariant('OKTA_DEFAULT'),
                               'signInPageTouchPointVariant': models.SignInPageTouchPointVariant('OKTA_DEFAULT'),
                               'errorPageTouchPointVariant': models.ErrorPageTouchPointVariant('OKTA_DEFAULT')}
        try:
            updated_theme, _, err = await client.update_brand_theme(brand_id, theme_id, update_theme_params)
            assert err is None
            assert updated_theme.primary_color_hex == '#ababab'
            assert updated_theme.secondary_color_hex == '#ebebeb'
            assert updated_theme.email_template_touch_point_variant == 'OKTA_DEFAULT'
            assert updated_theme.end_user_dashboard_touch_point_variant == 'OKTA_DEFAULT'
        finally:
            # restore previous theme settings
            restore_theme_params = {'primaryColorHex': theme.primary_color_hex,
                                    'secondaryColorHex': theme.secondary_color_hex,
                                    'emailTemplateTouchPointVariant': theme.email_template_touch_point_variant,
                                    'endUserDashboardTouchPointVariant': theme.end_user_dashboard_touch_point_variant,
                                    'signInPageTouchPointVariant': theme.sign_in_page_touch_point_variant,
                                    'errorPageTouchPointVariant': theme.error_page_touch_point_variant}
            _, _, err = await client.update_brand_theme(brand_id, theme_id, restore_theme_params)
            assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_delete_brand_theme_background_image(self, fs):
        client = MockOktaClient(fs)
        brands, _, err = await client.list_brands()
        assert err is None
        brand_id = brands[0].id
        themes, _, err = await client.list_brand_themes(brand_id)
        assert err is None
        theme_id = themes[0].id
        _, err = await client.delete_brand_theme_background_image(brand_id, theme_id)
        assert err is None

    # skip test until solution with writing cassette is found
    @pytest.mark.skip
    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_upload_brand_theme_background_image(self, fs):
        client = MockOktaClient(fs)
        brands, _, err = await client.list_brands()
        assert err is None
        brand_id = brands[0].id
        themes, _, err = await client.list_brand_themes(brand_id)
        assert err is None
        theme_id = themes[0].id
        try:
            image = open(f'tests/integration/data/brand_theme_background.png', 'rb')
            image_upload_resp, _, err = await client.upload_brand_theme_background_image(brand_id, theme_id, image)
            assert err is None
            assert isinstance(image_upload_resp, models.ImageUploadResponse)
        finally:
            _, err = await client.delete_brand_theme_background_image(brand_id, theme_id)
            assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_delete_brand_theme_favicon(self, fs):
        client = MockOktaClient(fs)
        brands, _, err = await client.list_brands()
        assert err is None
        brand_id = brands[0].id
        themes, _, err = await client.list_brand_themes(brand_id)
        assert err is None
        theme_id = themes[0].id
        _, err = await client.delete_brand_theme_favicon(brand_id, theme_id)
        assert err is None

    # skip test until solution with writing cassette is found
    @pytest.mark.skip
    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_upload_brand_theme_favicon(self, fs):
        client = MockOktaClient(fs)
        brands, _, err = await client.list_brands()
        assert err is None
        brand_id = brands[0].id
        themes, _, err = await client.list_brand_themes(brand_id)
        assert err is None
        theme_id = themes[0].id
        try:
            image = open(f'tests/integration/data/brand_theme_favicon.ico', 'rb')
            image_upload_resp, _, err = await client.upload_brand_theme_favicon(brand_id, theme_id, image)
            assert err is None
            assert isinstance(image_upload_resp, models.ImageUploadResponse)
        finally:
            _, err = await client.delete_brand_theme_favicon(brand_id, theme_id)
            assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_delete_brand_theme_logo(self, fs):
        client = MockOktaClient(fs)
        brands, _, err = await client.list_brands()
        assert err is None
        brand_id = brands[0].id
        themes, _, err = await client.list_brand_themes(brand_id)
        assert err is None
        theme_id = themes[0].id
        _, err = await client.delete_brand_theme_logo(brand_id, theme_id)
        assert err is None

    # skip test until solution with writing cassette is found
    @pytest.mark.skip
    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_upload_brand_theme_logo(self, fs):
        client = MockOktaClient(fs)
        brands, _, err = await client.list_brands()
        assert err is None
        brand_id = brands[0].id
        themes, _, err = await client.list_brand_themes(brand_id)
        assert err is None
        theme_id = themes[0].id
        try:
            image = open(f'tests/integration/data/brand_theme_logo.png', 'rb')
            image_upload_resp, _, err = await client.upload_brand_theme_logo(brand_id, theme_id, image)
            assert err is None
            assert isinstance(image_upload_resp, models.ImageUploadResponse)
        finally:
            _, err = await client.delete_brand_theme_logo(brand_id, theme_id)
            assert err is None



import pytest
from tests.mocks import MockOktaClient
import okta.models as models
from okta.errors.okta_api_error import OktaAPIError


class TestSubscriptionResource:
    """
    Integration Tests for the Subscription Resource
    """

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_list_role_subsription(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)
        resp, _, err = await client.list_role_subscriptions('SUPER_ADMIN')
        assert len(resp) > 0
        for item in resp:
            assert isinstance(item, models.Subscription)
            assert isinstance(item.status, models.SubscriptionStatus)
            assert isinstance(item.notification_type, models.NotificationType)

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_get_role_subscription_by_notification_type(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)
        notification_type = models.NotificationType('OKTA_ISSUE')
        resp, _, err = await client.get_role_subscription_by_notification_type('SUPER_ADMIN', notification_type)
        assert isinstance(resp, models.Subscription)
        assert resp.notification_type == models.NotificationType('OKTA_ISSUE')

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_subscribe_unsubscribe_role_by_notification_type(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)
        notification_type = models.NotificationType('OKTA_ISSUE')
        _, err = await client.subscribe_role_subscription_by_notification_type('SUPER_ADMIN', notification_type)
        assert err is None

        resp, _, err = await client.get_role_subscription_by_notification_type('SUPER_ADMIN', notification_type)
        assert isinstance(resp, models.Subscription)
        assert resp.notification_type == models.NotificationType('OKTA_ISSUE')
        assert resp.status == models.SubscriptionStatus('SUBSCRIBED')

        try:
            _, err = await client.unsubscribe_role_subscription_by_notification_type('SUPER_ADMIN', notification_type)
            assert err is None
            resp, _, err = await client.get_role_subscription_by_notification_type('SUPER_ADMIN', notification_type)
            assert isinstance(resp, models.Subscription)
            assert resp.notification_type == models.NotificationType('OKTA_ISSUE')
            assert resp.status == models.SubscriptionStatus('UNSUBSCRIBED')

        finally:
            # restore subscription
            _, err = await client.subscribe_role_subscription_by_notification_type('SUPER_ADMIN', notification_type)
            assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_subscribe_unsubscribe_user(self, fs):
        # Instantiate Mock Client
        client = MockOktaClient(fs)

        users, _, err = await client.list_users()
        assert err is None
        user = users[0]

        notification_type = models.NotificationType('OKTA_ISSUE')
        _, err = await client.subscribe_user_subscription_by_notification_type(user.id, notification_type)
        assert err is None

        resp, _, err = await client.get_user_subscription_by_notification_type(user.id, notification_type)
        assert resp.notification_type == models.NotificationType('OKTA_ISSUE')
        assert resp.status == models.SubscriptionStatus('SUBSCRIBED')

        _, err = await client.unsubscribe_user_subscription_by_notification_type(user.id, notification_type)
        assert err is None

        resp, _, err = await client.get_user_subscription_by_notification_type(user.id, notification_type)
        assert resp.notification_type == models.NotificationType('OKTA_ISSUE')
        assert resp.status == models.SubscriptionStatus('UNSUBSCRIBED')



import copy
import os
import pytest
from datetime import datetime
from tests.mocks import MockOktaClient
from okta.models import (OrgSetting,
                         OrgContactType,
                         OrgContactTypeObj,
                         OrgContactUser,
                         OrgPreferences,
                         OrgOktaCommunicationSetting,
                         OrgOktaSupportSettingsObj,
                         OrgOktaSupportSetting,
                         UserIdString)


class TestOrgResource:
    """
    Integration Tests for the Org Resource
    """

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_get_org_settings(self, fs):
        client = MockOktaClient(fs)
        org_settings, _, err = await client.get_org_settings()
        assert err is None
        assert isinstance(org_settings, OrgSetting)
        assert org_settings.status == 'ACTIVE'

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_partial_setting_update(self, fs):
        client = MockOktaClient(fs)
        get_org_settings, _, err = await client.get_org_settings()
        assert err is None

        updated_setting = {'supportPhoneNumber': '1234567890'}
        try:
            updated_org_settings, _, err = await client.partial_update_org_setting(updated_setting)
            assert err is None
            assert updated_org_settings.support_phone_number == '1234567890'
        finally:
            updated_setting = {'supportPhoneNumber': get_org_settings.support_phone_number}
            updated_org_settings, _, err = await client.partial_update_org_setting(updated_setting, keep_empty_params=True)
            assert err is None
            assert updated_org_settings.support_phone_number == get_org_settings.support_phone_number

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_setting_update(self, fs):
        client = MockOktaClient(fs)
        get_org_settings, resp, err = await client.get_org_settings()

        new_org_settings = copy.deepcopy(get_org_settings)
        new_org_settings.support_phone_number = '1234567890'
        new_org_settings.company_name = 'NewOrgName'
        try:
            updated_org_settings, _, err = await client.update_org_setting(new_org_settings)
            assert err is None
            assert updated_org_settings.support_phone_number == '1234567890'
            assert updated_org_settings.company_name == 'NewOrgName'

        finally:
            updated_org_settings, _, err = await client.update_org_setting(get_org_settings, keep_empty_params=True)
            assert err is None
            assert updated_org_settings.support_phone_number == get_org_settings.support_phone_number
            assert updated_org_settings.company_name == get_org_settings.company_name

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_get_org_contact_types(self, fs):
        client = MockOktaClient(fs)
        org_contact_types, _, err = await client.get_org_contact_types()
        assert err is None
        for item in org_contact_types:
            assert isinstance(item, OrgContactTypeObj)
            assert isinstance(item.contact_type, OrgContactType)

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_get_org_contact_user(self, fs):
        client = MockOktaClient(fs)
        contact_type = OrgContactType('BILLING')
        org_contact_user, _, err = await client.get_org_contact_user(contact_type)
        assert err is None
        assert isinstance(org_contact_user, OrgContactUser)
        assert org_contact_user.user_id is not None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_update_org_contact_user(self, fs):
        client = MockOktaClient(fs)
        contact_type = OrgContactType('BILLING')
        org_contact_user, _, err = await client.get_org_contact_user(contact_type)
        assert err is None

        new_contact_type = OrgContactType('TECHNICAL')
        try:
            updated_user, _, err = await client.update_org_contact_user(new_contact_type,
                                                                        UserIdString({'userId': org_contact_user.user_id}))
            assert err is None
        finally:
            updated_user, _, err = await client.update_org_contact_user(contact_type,
                                                                        UserIdString({'userId': org_contact_user.user_id}))
            assert err is None

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_get_org_preferences(self, fs):
        client = MockOktaClient(fs)
        org_preferences, _, err = await client.get_org_preferences()
        assert err is None
        assert isinstance(org_preferences, OrgPreferences)
        assert isinstance(org_preferences.show_end_user_footer, bool)

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_hide_okta_ui_footer(self, fs):
        client = MockOktaClient(fs)
        org_preferences, _, err = await client.hide_okta_ui_footer()
        assert err is None
        assert isinstance(org_preferences, OrgPreferences)
        assert not org_preferences.show_end_user_footer

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_show_okta_ui_footer(self, fs):
        client = MockOktaClient(fs)
        org_preferences, _, err = await client.show_okta_ui_footer()
        assert err is None
        assert isinstance(org_preferences, OrgPreferences)
        assert org_preferences.show_end_user_footer

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_get_okta_communication_settings(self, fs):
        client = MockOktaClient(fs)
        org_communication_setting, _, err = await client.get_okta_communication_settings()
        assert err is None
        assert isinstance(org_communication_setting, OrgOktaCommunicationSetting)
        assert isinstance(org_communication_setting.opt_out_email_users, bool)

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_opt_in_users_to_okta_communication_emails(self, fs):
        client = MockOktaClient(fs)
        org_communication_setting, _, err = await client.opt_in_users_to_okta_communication_emails()
        assert err is None
        assert isinstance(org_communication_setting, OrgOktaCommunicationSetting)
        assert not org_communication_setting.opt_out_email_users

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_opt_out_users_from_okta_communication_emails(self, fs):
        client = MockOktaClient(fs)
        org_communication_setting, _, err = await client.opt_out_users_from_okta_communication_emails()
        assert err is None
        assert isinstance(org_communication_setting, OrgOktaCommunicationSetting)
        assert org_communication_setting.opt_out_email_users

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_get_okta_support_settings(self, fs):
        client = MockOktaClient(fs)
        org_okta_support_setting, _, err = await client.get_org_okta_support_settings()
        assert err is None
        assert isinstance(org_okta_support_setting, OrgOktaSupportSettingsObj)
        assert isinstance(org_okta_support_setting.support, OrgOktaSupportSetting)

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_grant_revoke_okta_support(self, fs):
        client = MockOktaClient(fs)
        try:
            org_okta_support_setting, _, err = await client.grant_okta_support()
            assert err is None
            assert isinstance(org_okta_support_setting, OrgOktaSupportSettingsObj)
            assert isinstance(org_okta_support_setting.support, OrgOktaSupportSetting)
            assert org_okta_support_setting.support == 'ENABLED'
        finally:
            org_okta_support_setting, _, err = await client.revoke_okta_support()
            assert err is None
            assert isinstance(org_okta_support_setting, OrgOktaSupportSettingsObj)
            assert isinstance(org_okta_support_setting.support, OrgOktaSupportSetting)
            assert org_okta_support_setting.support == 'DISABLED'

    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_extend_okta_support(self, fs):
        client = MockOktaClient(fs)
        try:
            org_okta_support_setting, _, err = await client.grant_okta_support()
            assert err is None
            assert org_okta_support_setting.support == 'ENABLED'
            extended_org_okta_support_setting, _, err = await client.extend_okta_support()
            assert err is None
            date1 = datetime.strptime(org_okta_support_setting.expiration, "%Y-%m-%dT%H:%M:%S.%fZ")
            date2 = datetime.strptime(extended_org_okta_support_setting.expiration, "%Y-%m-%dT%H:%M:%S.%fZ")
            # should be 24h
            assert round((date2 - date1).total_seconds() / 3600) == 24
        finally:
            org_okta_support_setting, _, err = await client.revoke_okta_support()
            assert err is None

    # skip test until solution with writing cassette is found
    @pytest.mark.skip
    @pytest.mark.vcr()
    @pytest.mark.asyncio
    async def test_upload_org_logo(self, fs):
        client = MockOktaClient(fs)
        fs.pause()
        logo = open(f'tests/integration/data/logo.png', 'rb')
        _, err = await client.update_org_logo(logo)
        fs.resume()
        assert err is None



"""
Copyright 2020 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from okta.models.user_schema\
    import UserSchema
from okta.utils import format_url
from okta.api_client import APIClient


class UserSchemaClient(APIClient):
    """
    A Client object for the UserSchema resource.
    """

    def __init__(self):
        self._base_url = ""

    async def get_application_user_schema(
            self, appInstanceId,
            keep_empty_params=True
    ):
        """
        Fetches the Schema for an App User
        Args:
            app_instance_id {str}
        Returns:
            UserSchema
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/meta/schemas/apps/{appInstanceId}/default
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, UserSchema)

        if error:
            return (None, response, error)

        try:
            result = UserSchema(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_application_user_profile(
            self, appInstanceId, user_schema,
            keep_empty_params=True
    ):
        """
        Partial updates on the User Profile properties of the A
        pplication User Schema.
        Args:
            app_instance_id {str}
            {user_schema}
        Returns:
            UserSchema
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/meta/schemas/apps/{appInstanceId}/default
            """)

        if isinstance(user_schema, dict):
            body = user_schema
        else:
            body = user_schema.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, UserSchema)

        if error:
            return (None, response, error)

        try:
            result = UserSchema(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def get_user_schema(
            self, schemaId,
            keep_empty_params=True
    ):
        """
        Fetches the schema for a Schema Id.
        Args:
            schema_id {str}
        Returns:
            UserSchema
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/meta/schemas/user/{schemaId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, UserSchema)

        if error:
            return (None, response, error)

        try:
            result = UserSchema(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def update_user_profile(
            self, schemaId, user_schema,
            keep_empty_params=True
    ):
        """
        Partial updates on the User Profile properties of the u
        ser schema.
        Args:
            schema_id {str}
            {user_schema}
        Returns:
            UserSchema
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/meta/schemas/user/{schemaId}
            """)

        if isinstance(user_schema, dict):
            body = user_schema
        else:
            body = user_schema.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, UserSchema)

        if error:
            return (None, response, error)

        try:
            result = UserSchema(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)



"""
Copyright 2020 - Present Okta, Inc.

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""

# AUTO-GENERATED! DO NOT EDIT FILE DIRECTLY
# SEE CONTRIBUTOR DOCUMENTATION

from okta.models.session\
    import Session
from okta.utils import format_url
from okta.api_client import APIClient


class SessionClient(APIClient):
    """
    A Client object for the Session resource.
    """

    def __init__(self):
        self._base_url = ""

    async def create_session(
            self, create_session_request,
            keep_empty_params=False
    ):
        """
        Creates a new session for a user with a valid session t
        oken. Use this API if, for example, you want to set the
        session cookie yourself instead of allowing Okta to se
        t it, or want to hold the session ID in order to delete
        a session via the API instead of visiting the logout U
        RL.
        Args:
            {create_session_request}
        Returns:
            Session
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/sessions
            """)

        if isinstance(create_session_request, dict):
            body = create_session_request
        else:
            body = create_session_request.as_dict()
        headers = {
            "Accept": "application/json",
            "Content-Type": "application/json"
        }
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Session)

        if error:
            return (None, response, error)

        try:
            result = Session(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def end_session(
            self, sessionId,
            keep_empty_params=False
    ):
        """
        Method for
        /api/v1/sessions/{sessionId}
        Args:
            session_id {str}
        """
        http_method = "delete".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/sessions/{sessionId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, error)

        response, error = await self._request_executor\
            .execute(request)

        if error:
            return (response, error)

        return (response, None)

    async def get_session(
            self, sessionId,
            keep_empty_params=False
    ):
        """
        Get details about a session.
        Args:
            session_id {str}
        Returns:
            Session
        """
        http_method = "get".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/sessions/{sessionId}
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Session)

        if error:
            return (None, response, error)

        try:
            result = Session(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)

    async def refresh_session(
            self, sessionId,
            keep_empty_params=False
    ):
        """
        Method for
        /api/v1/sessions/{sessionId}/lifecycle/refresh
        Args:
            session_id {str}
        Returns:
            Session
        """
        http_method = "post".upper()
        api_url = format_url(f"""
            {self._base_url}
            /api/v1/sessions/{sessionId}/lifecycle/refresh
            """)

        body = {}
        headers = {}
        form = {}

        request, error = await self._request_executor.create_request(
            http_method, api_url, body, headers, form, keep_empty_params=keep_empty_params
        )

        if error:
            return (None, None, error)

        response, error = await self._request_executor\
            .execute(request, Session)

        if error:
            return (None, response, error)

        try:
            result = Session(
                self.form_response_body(response.get_body())
            )
        except Exception as error:
            return (None, response, error)
        return (result, response, None)
